// LICENSE:  LGPL2.1
// Copyright (c) 2002 Tomoya Tokairin


#include <windows.h>
#include "oggdec.h"

HINSTANCE HoggDll = NULL;  

//---------------------------------------
// init
BOOL EXPORT oggdec_init(){

	typedef BOOL (WINAPI *PFUNC)();
	PFUNC pFunc;
	HINSTANCE hOgg = NULL,hVorbis = NULL;
	BOOL bRet = FALSE;

	if(HoggDll != NULL) return TRUE;

	if((hOgg = LoadLibrary("ogg.dll")) == NULL) goto L_ERR;
	if((hVorbis = LoadLibrary("vorbis.dll")) == NULL) goto L_ERR;

	FreeLibrary(hOgg);
	FreeLibrary(hVorbis);

	if((HoggDll = LoadLibrary("oggdec.dll")) == NULL) goto L_ERR;
	pFunc = (PFUNC)GetProcAddress(HoggDll,"oggdec_init");
	if(pFunc != NULL) bRet = (*pFunc)();
	
	return bRet;

L_ERR:
	if(hOgg) FreeLibrary(hOgg);
	if(hVorbis) FreeLibrary(hVorbis);
	if(HoggDll) FreeLibrary(HoggDll);
	HoggDll = NULL;

	return FALSE;
}

//---------------------------------------
// free
BOOL EXPORT oggdec_free(){

	typedef BOOL (WINAPI *PFUNC)();
	PFUNC pFunc;

	BOOL bRet = FALSE;

	if(HoggDll == NULL) return FALSE;
	
	pFunc = (PFUNC)GetProcAddress(HoggDll,"oggdec_free");
	if(pFunc != NULL) bRet = (*pFunc)();

	// dll ̊J
	FreeLibrary(HoggDll);
	HoggDll = NULL;
	
	return bRet;
}



//---------------------------------------
// callback
BOOL EXPORT oggdec_callback(OGGINPUT_MSG_PROC pProc){

	typedef BOOL (WINAPI *PFUNC)(OGGINPUT_MSG_PROC);
	static PFUNC pFunc = NULL;

	BOOL bRet = FALSE;

	if(HoggDll == NULL) return FALSE;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(HoggDll,"oggdec_callback");
	if(pFunc) bRet = (*pFunc)(pProc);
	
	return bRet;
}



//---------------------------------------
// open
BOOL EXPORT oggdec_open(const char* pszName, OGGINPUTINFO* pInfo){

	typedef BOOL (WINAPI *PFUNC)(const char*,OGGINPUTINFO*);
	static PFUNC pFunc = NULL;

	BOOL bRet = FALSE;

	if(HoggDll == NULL) return FALSE;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(HoggDll,"oggdec_open");
	if(pFunc) bRet = (*pFunc)(pszName,pInfo);
	
	return bRet;
}



//---------------------------------------
// close
BOOL EXPORT oggdec_close(){

	typedef BOOL (WINAPI *PFUNC)();
	static  PFUNC pFunc;

	BOOL bRet = FALSE;

	if(HoggDll == NULL) return FALSE;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(HoggDll,"oggdec_close");
	if(pFunc) bRet = (*pFunc)();
	
	return bRet;
}



//---------------------------------------
// seek
BOOL EXPORT oggdec_seek(DWORD dwMsec){

	typedef BOOL (WINAPI *PFUNC)(DWORD);
	static PFUNC pFunc = NULL;

	BOOL bRet = FALSE;

	if(HoggDll == NULL) return FALSE;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(HoggDll,"oggdec_seek");
	if(pFunc) bRet = (*pFunc)(dwMsec);
	
	return bRet;
}



//---------------------------------------
// play
BOOL EXPORT oggdec_play(){

	typedef BOOL (WINAPI *PFUNC)();
	static PFUNC pFunc = NULL;

	BOOL bRet = FALSE;

	if(HoggDll == NULL) return FALSE;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(HoggDll,"oggdec_play");
	if(pFunc) bRet = (*pFunc)();
	
	return(bRet);
}



//---------------------------------------
// stop
BOOL EXPORT oggdec_stop(){

	typedef BOOL (WINAPI *PFUNC)();
	static PFUNC pFunc = NULL;

	BOOL bRet = FALSE;

	if(HoggDll == NULL) return FALSE;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(HoggDll,"oggdec_stop");
	if(pFunc) bRet = (*pFunc)();
	
	return bRet;
}





//---------------------------------------
// pause
BOOL EXPORT oggdec_pause(){

	typedef BOOL (WINAPI *PFUNC)();
	static PFUNC pFunc = NULL;

	BOOL bRet = FALSE;

	if(HoggDll == NULL) return FALSE;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(HoggDll,"oggdec_pause");
	if(pFunc) bRet = (*pFunc)();
	
	return bRet;
}




//---------------------------------------
// restart
BOOL EXPORT oggdec_restart(){

	typedef BOOL (WINAPI *PFUNC)();
	static PFUNC pFunc = NULL;

	BOOL bRet = FALSE;

	if(HoggDll == NULL) return FALSE;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(HoggDll,"oggdec_restart");
	if(pFunc) bRet = (*pFunc)();
	
	return bRet;
}



//---------------------------------------
// getWaveData
DWORD EXPORT oggdec_getWave(short* pWaveL, short* pWaveR, DWORD dwMaxSize){

	typedef DWORD (WINAPI *PFUNC)(short*,short*,DWORD);
	static PFUNC pFunc = NULL;
	DWORD dwRet = 0;

	if(HoggDll == NULL) return 0;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(HoggDll,"oggdec_getWave");
	if(pFunc) dwRet = (*pFunc)(pWaveL,pWaveR,dwMaxSize);
	
	return dwRet;
}



//---------------------------------------
// setWaveOutDeviceId
void EXPORT oggdec_setWaveOutDeviceId(unsigned int uId){

	typedef void (WINAPI *PFUNC)(unsigned int);
	static PFUNC pFunc = NULL;

	if(HoggDll == NULL) return;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(HoggDll,"oggdec_setWaveOutDeviceId");
	if(pFunc) (*pFunc)(uId);

}




//---------------------------------------
// getVersion
long EXPORT oggdec_getVersion(char* lpszVer){

	typedef long (WINAPI *PFUNC)(char*);
	static PFUNC pFunc = NULL;
	long nRet = 0;

	if(HoggDll == NULL) return 0;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(HoggDll,"oggdec_getVersion");
	if(pFunc) nRet = (*pFunc)(lpszVer);

	return nRet;
}




//---------------------------------------
// getState
DWORD EXPORT oggdec_getState(LPDWORD lpdwMsec){

	typedef DWORD (WINAPI *PFUNC)(LPDWORD);
	static PFUNC pFunc = NULL;
	DWORD dwRet = 0;

	if(HoggDll == NULL) return 0;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(HoggDll,"oggdec_getState");
	if(pFunc) dwRet = (*pFunc)(lpdwMsec);
	
	return dwRet;
}



//EOF
