// LICENSE: LGPL
// Copyright (c) 2002 Tomoya Tokairin

#include <windows.h>
#include "vbmp3.h"

HINSTANCE Hmp3Dll = NULL;  

#define EXPORT WINAPI

//---------------------------------------
// init
BOOL EXPORT vbmp3_init(){

	typedef BOOL (WINAPI *PFUNC)();
	PFUNC pFunc;

	BOOL bRet = FALSE;

	if(Hmp3Dll != NULL) return FALSE;

	if((Hmp3Dll = LoadLibrary("vbmp3.dll")) == NULL) return FALSE;

	pFunc = (PFUNC)GetProcAddress(Hmp3Dll,"vbmp3_init");
	if(pFunc != NULL) bRet = (*pFunc)();
	
	return bRet;
}



//---------------------------------------
// free
BOOL EXPORT vbmp3_free(){

	typedef BOOL (WINAPI *PFUNC)();
	PFUNC pFunc;

	BOOL bRet = FALSE;

	if(Hmp3Dll == NULL) return FALSE;
	
	pFunc = (PFUNC)GetProcAddress(Hmp3Dll,"vbmp3_free");
	if(pFunc != NULL) bRet = (*pFunc)();

	// dll ̊J
	FreeLibrary(Hmp3Dll);
	Hmp3Dll = NULL;
	
	return bRet;
}



//---------------------------------------
// callback
BOOL EXPORT vbmp3_callback(INPUT_MSG_PROC pProc){

	typedef BOOL (WINAPI *PFUNC)(INPUT_MSG_PROC);
	static PFUNC pFunc = NULL;

	BOOL bRet = FALSE;

	if(Hmp3Dll == NULL) return FALSE;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(Hmp3Dll,"vbmp3_callback");
	if(pFunc) bRet = (*pFunc)(pProc);
	
	return bRet;
}



//---------------------------------------
// open
BOOL EXPORT vbmp3_open(const char* pszName, InputInfo* pInfo){

	typedef BOOL (WINAPI *PFUNC)(const char*,InputInfo*);
	static PFUNC pFunc = NULL;

	BOOL bRet = FALSE;

	if(Hmp3Dll == NULL) return FALSE;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(Hmp3Dll,"vbmp3_open");
	if(pFunc) bRet = (*pFunc)(pszName,pInfo);
	
	return bRet;
}



//---------------------------------------
// close
BOOL EXPORT vbmp3_close(){

	typedef BOOL (WINAPI *PFUNC)();
	static  PFUNC pFunc;

	BOOL bRet = FALSE;

	if(Hmp3Dll == NULL) return FALSE;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(Hmp3Dll,"vbmp3_close");
	if(pFunc) bRet = (*pFunc)();
	
	return bRet;
}



//---------------------------------------
// seek
BOOL EXPORT vbmp3_seek(int pos){

	typedef BOOL (WINAPI *PFUNC)(int);
	static PFUNC pFunc = NULL;

	BOOL bRet = FALSE;

	if(Hmp3Dll == NULL) return FALSE;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(Hmp3Dll,"vbmp3_seek");
	if(pFunc) bRet = (*pFunc)(pos);
	
	return bRet;
}


//---------------------------------------
// play
BOOL EXPORT vbmp3_play(){

	typedef BOOL (WINAPI *PFUNC)();
	static PFUNC pFunc = NULL;

	BOOL bRet = FALSE;

	if(Hmp3Dll == NULL) return FALSE;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(Hmp3Dll,"vbmp3_play");
	if(pFunc) bRet = (*pFunc)();
	
	return(bRet);
}



//---------------------------------------
// stop
BOOL EXPORT vbmp3_stop(){

	typedef BOOL (WINAPI *PFUNC)();
	static PFUNC pFunc = NULL;

	BOOL bRet = FALSE;

	if(Hmp3Dll == NULL) return FALSE;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(Hmp3Dll,"vbmp3_stop");
	if(pFunc) bRet = (*pFunc)();
	
	return bRet;
}



//---------------------------------------
// getWaveData
void EXPORT vbmp3_getWave(int* pWaveL, int* pWaveR){

	typedef void (WINAPI *PFUNC)(int*,int*);
	static PFUNC pFunc = NULL;

	if(Hmp3Dll == NULL) return;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(Hmp3Dll,"vbmp3_getWave");
	if(pFunc) (*pFunc)(pWaveL,pWaveR);
	
}



//---------------------------------------
// setWaveOutDeviceId
void EXPORT vbmp3_setWaveOutDeviceId(unsigned int id){

	typedef void (WINAPI *PFUNC)(unsigned int);
	static PFUNC pFunc = NULL;

	if(Hmp3Dll == NULL) return;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(Hmp3Dll,"vbmp3_setWaveOutDeviceId");
	if(pFunc) (*pFunc)(id);

}



int EXPORT vbmp3_getState(int* sec){

	int nRet;
	typedef int (WINAPI *PFUNC)(int*);
	static PFUNC pFunc = NULL;

	if(Hmp3Dll == NULL) return 0;
	
	if(!pFunc) pFunc = (PFUNC)GetProcAddress(Hmp3Dll,"vbmp3_getState");
	if(pFunc) nRet = (*pFunc)(sec);

	return nRet;
}



//EOF
